//--------------------------------------------------------------------------------
// BMVU
//----------------------------------------------------------------------------------

/*
definition d'un bmvu
num est le numero de la premiere varaible U
il y en a 16 (8 pour chaque coté) avec l'offset suivant
0 sel blocage
1 bp redition
2 bp sonnerie
3 bp annonce
4 bp test
5 sel semaphore
6 voy semaphore
7 voy voie libre
*/

//#define NB_BMVU        1

#define BMVU_OFF                  0
#define BMVU_A_OK_WAIT_SEM_OUVERT 1
#define BMVU_A_WAIT_AUBINAGE      2
#define BMVU_A_WAIT_SEM_FERME     3
#define BMVU_A_WAIT_ANNONCE       4
#define BMVU_A_WAIT_PASSAGE       5
#define BMVU_A_WAIT_REDDITION     6
#define BMVU_B_OK_WAIT_SEM_OUVERT 7
#define BMVU_B_WAIT_AUBINAGE      8
#define BMVU_B_WAIT_SEM_FERME     9
#define BMVU_B_WAIT_ANNONCE       10
#define BMVU_B_WAIT_PASSAGE       11
#define BMVU_B_WAIT_REDDITION     12

#define BMVU_CLICK_OFF 0
#define BMVU_CLICK_A   1
#define BMVU_CLICK_T   2
#define BMVU_CLICK_R   3
#define BMVU_CLICK_S   4

class C_bmvu
{
    public:
        byte u       = 80;  // variable U de depart u...u+15
        byte in_sem1 = 255;
        byte in_rel1 = 255;
        byte in_sem2 = 255;
        byte in_rel2 = 255;
        byte state   = BMVU_OFF;
        byte dcpt    = 0;
        byte click_a = BMVU_CLICK_OFF;
        byte click_b = BMVU_CLICK_OFF;
};
C_bmvu bmvus[NB_BMVU];

void bmvu_add(byte num, byte u, byte in_sem1, byte in_rel1, byte in_rel2, byte in_sem2)
{
    bmvus[num].u = u;
    bmvus[num].in_sem1 = in_sem1;
    bmvus[num].in_rel1 = in_rel1;
    bmvus[num].in_sem2 = in_sem2;
    bmvus[num].in_rel2 = in_rel2;
}


void bmvu_click(byte u)
{
    byte b;

    for(b = 0; b < NB_BMVU; b++)
    {
        if(u >= bmvus[b].u && u<= (bmvus[b].u + 15))
        {
            u -= bmvus[b].u;
            if(u==1) bmvus[b].click_a = BMVU_CLICK_R;
            if(u==2) bmvus[b].click_a = BMVU_CLICK_S;
            if(u==3) bmvus[b].click_a = BMVU_CLICK_A;
            if(u==4) bmvus[b].click_a = BMVU_CLICK_T;
            if(u==(8+1)) bmvus[b].click_b = BMVU_CLICK_R;
            if(u==(8+2)) bmvus[b].click_b = BMVU_CLICK_S;
            if(u==(8+3)) bmvus[b].click_b = BMVU_CLICK_A;
            if(u==(8+4)) bmvus[b].click_b = BMVU_CLICK_T;
            if(u==0 || u==5 || u==(0+8) || u==(5+8)) user_set_u(u+bmvus[b].u, 1-user_get_u(u+bmvus[b].u)); //variable u des selecteurs
            u += bmvus[b].u;
        }
    }
}


void bmvu_update(void)
{
    byte b, u0;
    byte click_a, click_b;
    C_bmvu* p_bmvu;

    for(b = 0; b < NB_BMVU; b++)
    {
        yield();
      
        p_bmvu = &bmvus[b];

        click_a = p_bmvu->click_a; p_bmvu->click_a = BMVU_CLICK_OFF;
        click_b = p_bmvu->click_b; p_bmvu->click_b = BMVU_CLICK_OFF;
        u0 = p_bmvu->u;

        // evolution de la state machine
        if(p_bmvu->state == BMVU_OFF)
        {
            if(p_bmvu->dcpt) //ancienne sonnerie active
            {
                p_bmvu->dcpt--;
                if(p_bmvu->dcpt == 0)
                {
                    user_set_u(u0 + 2    , 0);  // S
                    user_set_u(u0 + 2 + 8, 0);  // S
                }
            }

            if((user_get_u(u0 + 0)==0) && (user_get_u(u0 + 8)==0)) // pas de blocage
            if( user_get_u(u0 + 5)==0 &&  user_get_u(u0 + 5 + 8)==0) // semaphore ferme
            {
                if(click_a == BMVU_CLICK_T)
                {
                    p_bmvu->state = BMVU_A_OK_WAIT_SEM_OUVERT;
                    p_bmvu->dcpt = 40;  //40*250ms=10s
                    user_set_u(u0 + 4, 1);       // T
                }
                else if(click_b == BMVU_CLICK_T)
                {
                    p_bmvu->state = BMVU_B_OK_WAIT_SEM_OUVERT;
                    p_bmvu->dcpt = 40;
                    user_set_u(u0 + 4 + 8, 1);   // T
                }
            }
        }

        if(p_bmvu->state == BMVU_A_OK_WAIT_SEM_OUVERT)
        {
            p_bmvu->dcpt--;
            if(p_bmvu->dcpt == 0)
            {
                p_bmvu->state = BMVU_OFF;
                user_set_u(u0 + 4,  0);
            }
            else if(user_get_u(u0 + 5) == 1)
            {
                p_bmvu->state = BMVU_A_WAIT_AUBINAGE;
                user_set_u(u0 + 4,  0);
            }
        }

        if(p_bmvu->state == BMVU_A_WAIT_AUBINAGE)
        {
            if(p_bmvu->in_sem1 == 255)            p_bmvu->state = BMVU_A_WAIT_SEM_FERME;
            else if(user_get_in(p_bmvu->in_sem1)) p_bmvu->state = BMVU_A_WAIT_SEM_FERME;
        }

        if(p_bmvu->state == BMVU_A_WAIT_SEM_FERME)
        {
            if(user_get_u(u0 + 5)==0) p_bmvu->state = BMVU_A_WAIT_ANNONCE;
        }

        if(p_bmvu->state == BMVU_A_WAIT_ANNONCE) if(click_a == BMVU_CLICK_A) if(user_get_u(u0 + 5)==0)
        {
            p_bmvu->state = BMVU_A_WAIT_PASSAGE;
            user_set_u(u0 + 3    , 1);   // A
            user_set_u(u0 + 1 + 8, 1);   // R
            user_set_u(u0 + 2 + 8, 1);   // S
        }

        if(p_bmvu->state == BMVU_A_WAIT_PASSAGE)
        {
            if(p_bmvu->in_rel2 == 255)            p_bmvu->state = BMVU_A_WAIT_REDDITION;
            else if(user_get_in(p_bmvu->in_rel2)) p_bmvu->state = BMVU_A_WAIT_REDDITION;
        }

        if(p_bmvu->state == BMVU_A_WAIT_REDDITION) if(click_b == BMVU_CLICK_R)
        {
            p_bmvu->state = BMVU_OFF;
            user_set_u(u0 + 3    , 0);   // A
            user_set_u(u0 + 1 + 8, 0);   // R
            user_set_u(u0 + 2 + 8, 0);   // S
            user_set_u(u0 + 2    , 1);   // S
            p_bmvu->dcpt = 8;    // 2s
        }

        if(p_bmvu->state == BMVU_B_OK_WAIT_SEM_OUVERT)
        {
            p_bmvu->dcpt--;
            if(p_bmvu->dcpt == 0)
            {
                p_bmvu->state = BMVU_OFF;
                user_set_u(u0 + 4 + 8, 0);
            }
            else if(user_get_u(u0 + 5 + 8) == 1)  // #SHEM
            {
                p_bmvu->state = BMVU_B_WAIT_AUBINAGE;
                user_set_u(u0 + 4 + 8,  0);
            }
        }

        if(p_bmvu->state == BMVU_B_WAIT_AUBINAGE)
        {
            if(p_bmvu->in_sem2 == 255)            p_bmvu->state = BMVU_B_WAIT_SEM_FERME;
            else if(user_get_in(p_bmvu->in_sem2)) p_bmvu->state = BMVU_B_WAIT_SEM_FERME;
        }

        if(p_bmvu->state == BMVU_B_WAIT_SEM_FERME)
        {
            if(user_get_u(u0 + 5 + 8)==0) p_bmvu->state = BMVU_B_WAIT_ANNONCE; // SHEM
        }

        if(p_bmvu->state == BMVU_B_WAIT_ANNONCE) if(click_b == BMVU_CLICK_A) if(user_get_u(u0 + 5 + 8)==0)
        {
            p_bmvu->state = BMVU_B_WAIT_PASSAGE;
            user_set_u(u0 + 3 + 8, 1);   // A
            user_set_u(u0 + 1    , 1);   // R
            user_set_u(u0 + 2    , 1);   // S
        }

        if(p_bmvu->state == BMVU_B_WAIT_PASSAGE)
        {
            if(p_bmvu->in_rel1 == 255)            p_bmvu->state = BMVU_B_WAIT_REDDITION;
            else if(user_get_in(p_bmvu->in_rel1)) p_bmvu->state = BMVU_B_WAIT_REDDITION;
        }

        if(p_bmvu->state == BMVU_B_WAIT_REDDITION) if(click_a == BMVU_CLICK_R)
        {
            p_bmvu->state = BMVU_OFF;
            user_set_u(u0 + 3 + 8, 0);   // A
            user_set_u(u0 + 1    , 0);   // R
            user_set_u(u0 + 2    , 0);   // S
            user_set_u(u0 + 2 + 8, 1);   // S
            p_bmvu->dcpt = 8;   //2s
        }

        // mise a jour du semaphore
        if(p_bmvu->state == BMVU_A_WAIT_AUBINAGE) user_set_u(u0 + 7, user_get_u(u0 + 5)^1);
        else if((p_bmvu->state == BMVU_A_WAIT_SEM_FERME) && (p_bmvu->in_sem1 == 255)) user_set_u(u0 + 7, 0);
        else user_set_u(u0 + 7, 1);

        if(p_bmvu->state == BMVU_B_WAIT_AUBINAGE) user_set_u(u0 + 7 + 8, user_get_u(u0 + 5 + 8)^1);
        else if((p_bmvu->state == BMVU_B_WAIT_SEM_FERME) && (p_bmvu->in_sem2 == 255)) user_set_u(u0 + 7 + 8, 0);
        else user_set_u(u0 + 7 + 8, 1);

        // mise a jour du voyant VL
        if((p_bmvu->state == BMVU_OFF) && (user_get_u(u0 + 0) == 0) && (user_get_u(u0 + 0 + 8) == 0) )
        {
            user_set_u(u0 + 6, 1);  user_set_u(u0 + 6 + 8, 1); // #VL (on peut gagner une variable u en metant en commun les 2 voyants
        }
        else
        {
            user_set_u(u0 + 6, 0);  user_set_u(u0 + 6 + 8, 0);
        }

        // effacement de la sonnerie
        if(click_a == BMVU_CLICK_S) user_set_u(u0 + 2    , 0);
        if(click_b == BMVU_CLICK_S) user_set_u(u0 + 2 + 8, 0);

        //#if USB_MODE == 1
        //Serial.print("\np_bmvu->state = "); Serial.println(p_bmvu->state);
        //#endif
    }
}



//--------------------------------------------------------------------------------
// AUBINAGE
//----------------------------------------------------------------------------------

class C_aubinage
{
    public:
        byte u_cmd = 255;
        byte u_sig = 255;
        byte in = 255;
        byte old_cmd = 0;
        byte state = 0;   //1=aubiné
};
C_aubinage aubinages[NB_AUBINAGE];

void aubinage_add(byte num, byte u_cmd, byte u_sig, byte in)
{
    aubinages[num].u_cmd = u_cmd;
    aubinages[num].u_sig = u_sig;    
    aubinages[num].in    = in;        
}

void aubinage_update(void)
{
    for(byte a = 0; a < NB_AUBINAGE; a++)
    {
        yield();
      
        byte u_cmd = aubinages[a].u_cmd;
        byte u_sig = aubinages[a].u_sig;
        if(u_cmd == 255 || u_sig == 255) continue;
        
        byte cmd = user_get_u(u_cmd);
        if(aubinages[a].old_cmd != cmd) aubinages[a].state = aubinages[a].old_cmd = cmd;
        byte in = aubinages[a].in;      
        if(in != 255) if(user_get_in(in)) aubinages[a].state = 1;
        
        user_set_u(u_sig, aubinages[a].state);        
    }
}
